# Define: pe_postgresql::db
#
# This module creates database instances, a user, and grants that user
# privileges to the database.
#
# Since it requires class pe_postgresql::server, we assume to run all commands as the
# postgresql user against the local postgresql server.
#
# TODO: support an array of privileges for "grant"; currently only supports a single
#  privilege, which is pretty useless unless that privilege is "ALL"
#
# Parameters:
#   [*title*]       - postgresql database name.
#   [*user*]        - username to create and grant access.
#   [*password*]    - user's password.  may be md5-encoded, in the format returned by the "pe_postgresql_password"
#                            function in this module
#   [*charset*]     - database charset. defaults to 'utf8'
#   [*grant*]       - privilege to grant user. defaults to 'all'.
#   [*tablespace*]  - database tablespace. default to use the template database's tablespace.
#   [*locale*]      - locale for database. defaults to 'undef' (effectively 'C').
#   [*istemplate*]  - determines whether or not to define database as a template. defaults to false.
#
# Actions:
#
# Requires:
#
#   class pe_postgresql::server
#
# Sample Usage:
#
#  pe_postgresql::db { 'mydb':
#    user     => 'my_user',
#    password => 'password',
#    grant    => 'all'
#  }
#
define pe_postgresql::db (
  $user,
  $password,
  $charset    = $pe_postgresql::params::charset,
  $locale     = $pe_postgresql::params::locale,
  $grant      = 'ALL',
  $tablespace = undef,
  $istemplate = false
) {
  include pe_postgresql::params

  pe_postgresql::database { $name:
    # TODO: ensure is not yet supported
    #ensure    => present,
    charset    => $charset,
    tablespace => $tablespace,
    #provider  => 'postgresql',
    require    => Class['pe_postgresql::server'],
    locale     => $locale,
    istemplate => $istemplate,
  }

  if ! defined(Pe_postgresql::Database_user[$user]) {
    pe_postgresql::database_user { $user:
      # TODO: ensure is not yet supported
      #ensure         => present,
      password_hash   => $password,
      #provider       => 'postgresql',
      require         => Pe_postgresql::Database[$name],
    }
  }

  pe_postgresql::database_grant { "GRANT ${user} - ${grant} - ${name}":
    privilege       => $grant,
    db              => $name,
    role            => $user,
    #provider       => 'postgresql',
    require         => [Pe_postgresql::Database[$name], Pe_postgresql::Database_user[$user]],
  }

}
